@@ -2,7 +2,6 @@
 $description United Arab Emirates CDN hosting live content for various websites in The Middle East.
 $url alwasat.ly
 $url media.gov.kw
-$url rotana.net
 $type live
 $region various
 """
@@ -15,19 +14,12 @@ from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugin.api import validate
 from streamlink.stream.hls import HLSStream
 
+
 log = logging.getLogger(__name__)
 
 
-@pluginmatcher(re.compile(r"""
-    https?://(?:www\.)?
-    (
-        alwasat\.ly
-    |
-        media\.gov\.kw
-    |
-        rotana\.net
-    )
-""", re.VERBOSE))
+@pluginmatcher(name="alwasatly", pattern=re.compile(r"https?://(?:www\.)?alwasat\.ly"))
+@pluginmatcher(name="mediagovkw", pattern=re.compile(r"https?://(?:www\.)?media\.gov\.kw"))
 class HiPlayer(Plugin):
     DAI_URL = "https://pubads.g.doubleclick.net/ssai/event/{0}/streams"
 
@@ -46,8 +38,8 @@ class HiPlayer(Plugin):
                 ),
             ),
         )
-
         if not js_url:
+            log.error("Could not find JS URL")
             return
 
         log.debug(f"JS URL={js_url}")
@@ -55,9 +47,9 @@ class HiPlayer(Plugin):
         data = self.session.http.get(
             js_url,
             schema=validate.Schema(
-                re.compile(r"i\s*=\s*\[(.*)]\.join"),
+                re.compile(r"\[(?P<data>[^]]+)]\.join\([\"']{2}\)"),
                 validate.none_or_all(
-                    validate.get(1),
+                    validate.get("data"),
                     validate.transform(lambda s: re.sub(r"['\", ]", "", s)),
                     validate.transform(lambda s: base64.b64decode(s)),
                     validate.parse_json(),
@@ -73,13 +65,16 @@ class HiPlayer(Plugin):
                 ),
             ),
         )
+        if not data:
+            log.error("Could not find base64 encoded JSON data")
+            return
 
         hls_url = data["streamUrl"]
 
         if data["daiEnabled"]:
             log.debug("daiEnabled=true")
             hls_url = self.session.http.post(
-                self.DAI_URL.format(data['daiAssetKey']),
+                self.DAI_URL.format(data["daiAssetKey"]),
                 data={"api-key": data["daiApiKey"]},
                 schema=validate.Schema(
                     validate.parse_json(),
