@@ -1,17 +1,24 @@
+"""
+$description Sporting channel live stream owned by Sportal, a Bulgarian sports media website.
+$url sportal.bg
+$type live
+"""
+
 import logging
 import re
 
 from streamlink.plugin import Plugin, pluginmatcher
-from streamlink.stream import HLSStream
+from streamlink.stream.hls import HLSStream
+
 
 log = logging.getLogger(__name__)
 
 
 @pluginmatcher(re.compile(
-    r'https?://(?:www\.)?sportal\.bg/sportal_live_tv\.php'
+    r"https?://(?:www\.)?sportal\.bg/sportal_live_tv\.php",
 ))
 class Sportal(Plugin):
-    _hls_re = re.compile(r'''["'](?P<url>[^"']+\.m3u8[^"']*?)["']''')
+    _hls_re = re.compile(r"""["'](?P<url>[^"']+\.m3u8[^"']*?)["']""")
 
     def _get_streams(self):
         res = self.session.http.get(self.url)
@@ -19,9 +26,9 @@ class Sportal(Plugin):
         if not m:
             return
 
-        hls_url = m.group('url')
-        log.debug('URL={0}'.format(hls_url))
-        log.warning('SSL certificate verification is disabled.')
+        hls_url = m.group("url")
+        log.debug("URL={0}".format(hls_url))
+        log.warning("SSL certificate verification is disabled.")
         return HLSStream.parse_variant_playlist(
             self.session, hls_url, verify=False).items()
 
