@@ -3,6 +3,8 @@ $description Live TV channels and VODs from TVP, a Polish public, state-owned br
 $url tvp.info
 $url tvp.pl
 $type live, vod
+$metadata id
+$metadata title
 $notes Some VODs may be geo-restricted. Authentication is not supported.
 """
 
@@ -216,9 +218,9 @@ class TVP(Plugin):
             log.error("The content is not available in your region")
             return
 
-        for format in data.get("formats"):
-            if format.get("mimeType") == "application/x-mpegurl":
-                return HLSStream.parse_variant_playlist(self.session, format.get("url"))
+        for formatitem in data.get("formats"):
+            if formatitem.get("mimeType") == "application/x-mpegurl":
+                return HLSStream.parse_variant_playlist(self.session, formatitem.get("url"))
 
     def _get_streams(self):
         if self.matches["tvp_info"]:
