@@ -3,6 +3,10 @@ $description Global live-streaming and video hosting social platform owned by Go
 $url youtube.com
 $url youtu.be
 $type live, vod
+$metadata id
+$metadata author
+$metadata category
+$metadata title
 $notes Protected videos are not supported
 """
 
@@ -135,7 +139,7 @@ class YouTube(Plugin):
         schema = validate.Schema(
             {"playabilityStatus": {
                 "status": str,
-                validate.optional("reason"): str,
+                validate.optional("reason"): validate.any(str, None),
             }},
             validate.get("playabilityStatus"),
             validate.union_get("status", "reason"),
@@ -224,7 +228,7 @@ class YouTube(Plugin):
         best_audio_itag = None
 
         # Extract audio streams from the adaptive format list
-        for url, label, itag, mimeType in adaptive_formats:
+        for url, _label, itag, mimeType in adaptive_formats:
             if url is None:
                 continue
 
@@ -341,7 +345,8 @@ class YouTube(Plugin):
         if not data:
             return False
         status, reason = self._schema_playabilitystatus(data)
-        if status != "OK":
+        # assume that there's an error if reason is set (status will still be "OK" for some reason)
+        if status != "OK" or reason:
             if errorlog:
                 log.error(f"Could not get video info - {status}: {reason}")
             return False
