@@ -62,7 +62,7 @@ class ByteRangeOffset:
         return bytes_start, self._calc_end(bytes_start, byterange.range)
 
 
-class HLSStreamWriter(SegmentedStreamWriter):
+class HLSStreamWriter(SegmentedStreamWriter[Sequence, Response]):
     WRITE_CHUNK_SIZE = 8192
 
     reader: "HLSStreamReader"
@@ -284,7 +284,7 @@ class HLSStreamWriter(SegmentedStreamWriter):
             log.debug(f"Segment {sequence.num} complete")
 
 
-class HLSStreamWorker(SegmentedStreamWorker):
+class HLSStreamWorker(SegmentedStreamWorker[Sequence, Response]):
     reader: "HLSStreamReader"
     writer: "HLSStreamWriter"
     stream: "HLSStream"
@@ -366,7 +366,7 @@ class HLSStreamWorker(SegmentedStreamWorker):
             return sequences[-1].segment.duration
         if self.playlist_reload_time_override == "live-edge" and sequences:
             return sum(s.segment.duration for s in sequences[-max(1, self.live_edge - 1):])
-        if type(self.playlist_reload_time_override) is float and self.playlist_reload_time_override > 0:
+        if type(self.playlist_reload_time_override) is float and self.playlist_reload_time_override > 0:  # noqa: E721
             return self.playlist_reload_time_override
         if playlist.targetduration:
             return playlist.targetduration
@@ -477,13 +477,15 @@ class HLSStreamWorker(SegmentedStreamWorker):
                     log.info(f"Stopping stream early after {self.duration_limit}")
                     return
 
-                # End of stream
-                stream_end = self.playlist_end is not None and sequence.num >= self.playlist_end
-                if self.closed or stream_end:
+                if self.closed:  # pragma: no cover
                     return
 
                 self.playlist_sequence = sequence.num + 1
 
+            # End of stream
+            if self.closed or self.playlist_end is not None and (not queued or self.playlist_sequence > self.playlist_end):
+                return
+
             if queued:
                 self.playlist_sequences_last = now()
             elif self._segment_queue_timing_threshold_reached():
@@ -509,7 +511,7 @@ class HLSStreamWorker(SegmentedStreamWorker):
                     log.warning(f"Failed to reload playlist: {err}")
 
 
-class HLSStreamReader(FilteredStream, SegmentedStreamReader):
+class HLSStreamReader(FilteredStream, SegmentedStreamReader[Sequence, Response]):
     __worker__ = HLSStreamWorker
     __writer__ = HLSStreamWriter
 
